import json
import os
import sys

from common.variables import ENCODING, MAX_PACKAGE_LENGTH

sys.path.append(os.path.join(os.getcwd(), '..'))
from common.decorators import log


@log
def get_message(client):
    """
    Get and decode messages
    :param client: socket
    :return:
    """

    encoded_response = client.recv(MAX_PACKAGE_LENGTH)
    if isinstance(encoded_response, bytes):
        json_response = encoded_response.decode(ENCODING)
        response = json.loads(json_response)
        if isinstance(response, dict):
            return response
        raise ValueError
    raise ValueError


@log
def send_json_message(sock, message):
    """
    Modify dict to json and send it via provided socket
    :param sock: socket
    :param message: dict
    :return:
    """
    if not isinstance(message, dict):
        raise TypeError

    json_message = json.dumps(message)
    encoded_message = json_message.encode(ENCODING)
    sock.send(encoded_message)



from sqlalchemy import create_engine, Table, Column, Integer, String, Text, MetaData, DateTime
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import mapper, sessionmaker
from common.variables import *
import datetime

Base = create_engine


class ClientDatabase:
    Base = declarative_base()

    class KnownUsers(Base):
        __tablename__ = 'known_users'

        id = Column(Integer, primary_key=True)
        username = Column(String)

        def __init__(self, user):
            self.username = user

    class MessageHistory(Base):
        __tablename__ = 'message_history'

        id = Column(Integer, primary_key=True)
        from_user = Column(String)
        to_user = Column(String)
        message = Column(Text)
        date = Column(DateTime)

        def __init__(self, from_user, to_user, message):
            self.from_user = from_user
            self.to_user = to_user
            self.message = message
            self.date = datetime.datetime.now()

    class Contacts(Base):
        __tablename__ = 'contacts'

        id = Column(Integer, primary_key=True)
        name = Column(String)

        def __init__(self, contact):
            self.name = contact

    def __init__(self, name):
        self.database_engine = create_engine(f'sqlite:///client_{name}.db3',
                                             echo=False,
                                             pool_recycle=7200,
                                             connect_args={'check_same_thread': False})

        self.Base.metadata.create_all(self.database_engine)
        Session = sessionmaker(bind=self.database_engine)
        self.session = Session()

        self.session.query(self.Contacts).delete()
        self.session.commit()

    def add_contact(self, contact):
        if not self.session.query(self.Contacts).filter_by(name=contact).count():
            contact_row = self.Contacts(contact)
            self.session.add(contact_row)
            self.session.commit()

    def del_contact(self, contact):
        self.session.query(self.Contacts).filter_by(name=contact).delete()
        self.session.commit()

    def add_users(self, users_list):
        self.session.query(self.KnownUsers).delete()
        for user in users_list:
            user_row = self.KnownUsers(user)
            self.session.add(user_row)
        self.session.commit()

    def save_message(self, from_user, to_user, message):
        message_row = self.MessageHistory(from_user, to_user, message)
        self.session.add(message_row)
        self.session.commit()

    def get_contacts(self):
        return [contact[0] for contact in self.session.query(self.Contacts.name).all()]

    def get_users(self):
        return [user[0] for user in self.session.query(self.KnownUsers.username).all()]

    def check_user(self, user):
        if self.session.query(self.KnownUsers).filter_by(username=user).count():
            return True
        else:
            return False

    def check_contact(self, contact):
        if self.session.query(self.Contacts).filter_by(name=contact).count():
            return True
        else:
            return False

    def get_history(self, from_who=None, to_who=None):
        query = self.session.query(self.MessageHistory)
        if from_who:
            query = query.filter_by(from_user=from_who)
        if to_who:
            query = query.filter_by(to_user=to_who)
        return [(history_row.from_user, history_row.to_user, history_row.message, history_row.date)
                for history_row in query.all()]


# Debugging
if __name__ == '__main__':
    test_db = ClientDatabase('test1')
    for i in ['test3', 'test4', 'test5']:
        test_db.add_contact(i)
    test_db.add_contact('test4')
    test_db.add_users(['test1', 'test2', 'test3', 'test4', 'test5'])
    test_db.save_message('test1', 'test2',
                         f'Hi! This is the test message from {datetime.datetime.now()}!')
    test_db.save_message('test2', 'test1',
                         f'Hi! This this another test message from {datetime.datetime.now()}!')
    print(test_db.get_contacts())
    print(test_db.get_users())
    print(test_db.check_user('test1'))
    print(test_db.check_user('test10'))
    print(test_db.get_history('test2'))
    print(test_db.get_history(to_who='test2'))
    print(test_db.get_history('test3'))
    test_db.del_contact('test4')
    print(test_db.get_contacts())



import os
import sys

sys.path.append('../')
from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton, QApplication
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QStandardItemModel, QStandardItem

import logging
sys.path.append(os.path.join(os.getcwd(), '../'))
from logs import client_log_config

client_log = logging.getLogger('client_log')


class DelContactDialog(QDialog):
    def __init__(self, database):
        super().__init__()
        self.database = database

        self.setFixedSize(250, 120)
        self.setWindowTitle('Select a contact to delete')

        self.setAttribute(Qt.WA_DeleteOnClose)

        self.setModal(True)

        self.selector_label = QLabel('Select a contact to delete:', self)
        self.selector_label.setFixedSize(230, 20)
        self.selector_label.move(30, 10)

        self.selector = QComboBox(self)
        self.selector.setFixedSize(200, 20)
        self.selector.move(20, 40)

        self.selector.addItems(sorted(self.database.get_contacts()))

        self.btn_ok = QPushButton('Delete', self)
        self.btn_ok.setFixedSize(100, 30)
        self.btn_ok.move(20, 80)

        self.btn_cancel = QPushButton('Cancel', self)
        self.btn_cancel.setFixedSize(100, 30)
        self.btn_cancel.move(120, 80)
        self.btn_cancel.clicked.connect(self.close)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    from client_database import ClientDatabase
    database = ClientDatabase('test1')
    window = DelContactDialog(database)
    database.add_contact('test1')
    database.add_contact('test2')
    print(database.get_contacts())
    window.selector.addItems(sorted(database.get_contacts()))
    window.show()
    app.exec_()



import hashlib
import hmac
import json
import os
import sys
import threading
import socket
import time

import binascii
from PyQt5.QtCore import QObject, pyqtSignal

from common.utils import send_json_message, get_message
from common.variables import *
from common.errors import ServerError

import logging
sys.path.append(os.path.join(os.getcwd(), '..'))

client_log = logging.getLogger('client_log')
socket_lock = threading.Lock()


class ClientTransport(threading.Thread, QObject):
    new_message = pyqtSignal(str)
    connection_lost = pyqtSignal()
    message_205 = pyqtSignal()
    
    def __init__(self, ip_address, port, database, username, password, keys):
        threading.Thread.__init__(self)
        QObject.__init__(self)
        # DB object
        self.database = database
        self.username = username
        self.password = password
        self.keys = keys
        # Socket to work with server
        self.transport = None

        self.connection_init(ip_address, port)

        try:
            self.user_list_update()
            self.contacts_list_update()
        except OSError as err:
            if err.errno:
                client_log.critical('Connection lost')
                raise ServerError('Connection with server lost')
            client_log.error('Timeout of connection during user list update')
        except json.JSONDecodeError as err:
            client_log.critical('Connection lost')
            raise ServerError('Connection with server lost')
        self.running = True

    def connection_init(self, ip, port):
        self.transport = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.transport.settimeout(5)

        # Try to connect 5 times at least
        connected = False
        for i in range(5):
            client_log.info(f'Connection attempt №{i + 1}')
            try:
                self.transport.connect((ip, port))
            except (OSError, ConnectionRefusedError):
                pass
            else:
                connected = True
                break
            time.sleep(1)

        if not connected:
            client_log.critical('Cannot connect to the server')
            raise ServerError('Cannot connect to the server')

        client_log.debug('Starting auth dialog.')

        password_bytes = self.password.encode('utf-8')
        salt = self.username.lower().encode('utf-8')
        password_hash = hashlib.pbkdf2_hmac('sha512', password_bytes, salt, 10000)
        password_hash_string = binascii.hexlify(password_hash)

        client_log.debug(f'Passwd hash ready: {password_hash_string}')

        # Got pub_key and decode it from bytes
        pubkey = self.keys.publickey().export_key().decode('ascii')

        # Authorising on server
        with socket_lock:
            presense = {
                ACTION: PRESENCE,
                TIME: time.time(),
                USER: {
                    ACCOUNT_NAME: self.username,
                    PUBLIC_KEY: pubkey
                }
            }
            client_log.debug(f"Presense message = {presense}")

            try:
                send_json_message(self.transport, presense)
                ans = get_message(self.transport)
                client_log.debug(f'Server response = {ans}.')

                if RESPONSE in ans:
                    if ans[RESPONSE] == 400:
                        raise ServerError(ans[ERROR])
                    elif ans[RESPONSE] == 511:
                        ans_data = ans[DATA]
                        hash = hmac.new(password_hash_string, ans_data.encode('utf-8'), 'MD5')
                        digest = hash.digest()
                        my_ans = RESPONSE_511
                        my_ans[DATA] = binascii.b2a_base64(
                            digest).decode('ascii')
                        send_json_message(self.transport, my_ans)
                        self.process_server_ans(get_message(self.transport))
            except (OSError, json.JSONDecodeError) as err:
                client_log.debug(f'Connection error.', exc_info=err)
                raise ServerError('Authorization failed')

        client_log.info('Connected to the server')

    def create_presence(self):
        message = {
            ACTION: PRESENCE,
            TIME: time.time(),
            USER: {
                ACCOUNT_NAME: self.username
            }
        }
        client_log.debug(f'Presence {PRESENCE} created for client {self.username}')
        return message

    def process_server_ans(self, message):
        client_log.debug(f'Parse server answer: {message}')

        if RESPONSE in message:
            if message[RESPONSE] == 200:
                return
            elif message[RESPONSE] == 400:
                raise ServerError(f'{message[ERROR]}')
            elif message[RESPONSE] == 205:
                self.user_list_update()
                self.contacts_list_update()
                self.message_205.emit()
            else:
                client_log.debug(f'Unknown response code {message[RESPONSE]}')

        # If there is a message add to DB and send signal about new message
        elif ACTION in message \
                and message[ACTION] == MESSAGE \
                and SENDER in message \
                and DESTINATION in message \
                and MESSAGE_TEXT in message \
                and message[DESTINATION] == self.username:
            client_log.debug(f'Got message from user {message[SENDER]}:'
                         f'{message[MESSAGE_TEXT]}')
            self.database.save_message(message[SENDER], 'in', message[MESSAGE_TEXT])
            self.new_message.emit(message[SENDER])

    def contacts_list_update(self):
        client_log.debug(f'Request user\'s contacts list {self.name}')
        req = {
            ACTION: GET_CONTACTS,
            TIME: time.time(),
            USER: self.username
        }
        client_log.debug(f'Request {req} is ready')
        with socket_lock:
            send_json_message(self.transport, req)
            ans = get_message(self.transport)
        client_log.debug(f'Server answer received {ans}')
        if RESPONSE in ans and ans[RESPONSE] == 202:
            for contact in ans[LIST_INFO]:
                self.database.add_contact(contact)
        else:
            client_log.error('Cannot update user contacts list')

    def user_list_update(self):
        client_log.debug(f'Request known users list {self.username}')
        req = {
            ACTION: USERS_REQUEST,
            TIME: time.time(),
            ACCOUNT_NAME: self.username
        }
        with socket_lock:
            send_json_message(self.transport, req)
            ans = get_message(self.transport)
        if RESPONSE in ans and ans[RESPONSE] == 202:
            self.database.add_users(ans[LIST_INFO])
        else:
            client_log.error('Cannot update users list')

    def key_request(self, user):
        client_log.debug(f'Request public key for {user}')
        req = {
            ACTION: PUBLIC_KEY_REQUEST,
            TIME: time.time(),
            ACCOUNT_NAME: user
        }
        with socket_lock:
            send_json_message(self.transport, req)
            ans = get_message(self.transport)
        if RESPONSE in ans and ans[RESPONSE] == 511:
            return ans[DATA]
        else:
            client_log.error(f'Cannot get interlocutor key {user}.')

    def add_contact(self, contact):
        client_log.debug(f'Create contact {contact}')
        req = {
            ACTION: ADD_CONTACT,
            TIME: time.time(),
            USER: self.username,
            ACCOUNT_NAME: contact
        }
        with socket_lock:
            send_json_message(self.transport, req)
            self.process_server_ans(get_message(self.transport))

    def remove_contact(self, contact):
        client_log.debug(f'Delete contact {contact}')
        req = {
            ACTION: REMOVE_CONTACT,
            TIME: time.time(),
            USER: self.username,
            ACCOUNT_NAME: contact
        }
        with socket_lock:
            send_json_message(self.transport, req)
            self.process_server_ans(get_message(self.transport))

    def transport_shutdown(self):
        self.running = False
        message = {
            ACTION: EXIT,
            TIME: time.time(),
            ACCOUNT_NAME: self.username
        }
        with socket_lock:
            try:
                send_json_message(self.transport, message)
            except OSError:
                pass
        client_log.debug('Transport is finishing its work.')
        time.sleep(0.5)

    def send_message(self, to, message):
        message_dict = {
            ACTION: MESSAGE,
            SENDER: self.username,
            DESTINATION: to,
            TIME: time.time(),
            MESSAGE_TEXT: message
        }
        client_log.debug(f'Message dict is prepared: {message_dict}')

        # Wait until socket is free
        with socket_lock:
            send_json_message(self.transport, message_dict)
            self.process_server_ans(get_message(self.transport))
            client_log.info(f'Send message from user {to}')

    def run(self):
        client_log.debug('Start process - Message receiver')
        while self.running:
            # Wait for a sec and try to get a socket again, need for freeing socket for message sender
            time.sleep(1)
            with socket_lock:
                try:
                    self.transport.settimeout(0.5)
                    message = get_message(self.transport)
                except OSError as err:
                    if err.errno:
                        client_log.critical(f'Connection lost')
                        self.running = False
                        self.connection_lost.emit()
                except (ConnectionError, ConnectionAbortedError,
                        ConnectionResetError, json.JSONDecodeError, TypeError):
                    client_log.debug(f'Connection lost')
                    self.running = False
                    self.connection_lost.emit()
                else:
                    client_log.debug(f'Got message from server: {message}')
                    self.process_server_ans(message)
                finally:
                    self.transport.settimeout(5)



import logging
import os
print(os.getcwd())
DIR = f'{os.getcwd()}/logs' if __name__ != '__main__' else f'{os.getcwd()}'

client_log = logging.getLogger('client_log')
client_log.setLevel(logging.DEBUG)

formatter = logging.Formatter("%(asctime)s %(levelname)-10s  %(module)15s  %(message)s")

file_handler = logging.FileHandler(f'{DIR}/logging/client_log.log', encoding='utf-8')
file_handler.setLevel(logging.DEBUG)
file_handler.setFormatter(formatter)

client_log.addHandler(file_handler)


if __name__ == '__main__':
    client_log.debug('Debug msg')
    client_log.info('Info msg')
    client_log.warning('Warning msg')
    client_log.error('Error msg')
    client_log.critical('Crit msg')



import os
import sys

from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton, QApplication
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QStandardItemModel, QStandardItem

import logging
sys.path.append(os.path.join(os.getcwd(), '../'))
from logs import client_log_config

client_log = logging.getLogger('client_log')


class AddContactDialog(QDialog):
    def __init__(self, transport, database):
        super().__init__()
        self.transport = transport
        self.database = database

        self.setFixedSize(350, 120)
        self.setWindowTitle('Pick contact for adding:')

        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setModal(True)

        self.selector_label = QLabel('Pick contact for adding:', self)
        self.selector_label.setFixedSize(200, 20)
        self.selector_label.move(10, 0)

        self.selector = QComboBox(self)
        self.selector.setFixedSize(200, 20)
        self.selector.move(10, 30)

        self.btn_refresh = QPushButton('Update list', self)
        self.btn_refresh.setFixedSize(100, 30)
        self.btn_refresh.move(60, 60)

        self.btn_ok = QPushButton('Add', self)
        self.btn_ok.setFixedSize(100, 30)
        self.btn_ok.move(230, 20)

        self.btn_cancel = QPushButton('Cancel', self)
        self.btn_cancel.setFixedSize(100, 30)
        self.btn_cancel.move(230, 60)
        self.btn_cancel.clicked.connect(self.close)

        self.possible_contacts_update()
        self.btn_refresh.clicked.connect(self.update_possible_contacts)

    def possible_contacts_update(self):
        self.selector.clear()
        contacts_list = set(self.database.get_contacts())
        users_list = set(self.database.get_users())
        users_list.remove(self.transport.username)
        self.selector.addItems(users_list - contacts_list)

    def update_possible_contacts(self):
        try:
            self.transport.user_list_update()
        except OSError:
            pass
        else:
            client_log.debug('User list updated from server')
            self.possible_contacts_update()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    from client_database import ClientDatabase
    database = ClientDatabase('test1')
    from transport import ClientTransport
    transport = ClientTransport('127.0.0.1', 7777, database, 'test1')
    window = AddContactDialog(transport, database)
    window.show()
    app.exec_()



import argparse
import os
import sys
import threading
from Crypto.PublicKey import RSA
from PyQt5.QtWidgets import QApplication, QMessageBox

from main_window import ClientMainWindow
from start_dialog import UserNameDialog
from transport import ClientTransport
from common.errors import ServerError
from common.variables import DEFAULT_SERVER_ADDRESS, DEFAULT_SERVER_PORT
from common.decorators import log
from client_database import ClientDatabase

import logging
sys.path.append(os.path.join(os.getcwd(), '..'))

client_log = logging.getLogger('client_log')

sock_lock = threading.Lock()
database_lock = threading.Lock()

@log
def arg_parser():
    """Parse command's arguments and prepare them to use"""
    parser = argparse.ArgumentParser()
    parser.add_argument('addr', default=DEFAULT_SERVER_ADDRESS, nargs='?')
    parser.add_argument('port', default=DEFAULT_SERVER_PORT, type=int, nargs='?')
    parser.add_argument('-n', '--name', default=None, nargs='?')
    parser.add_argument('-p', '--password', default='', nargs='?')
    namespace = parser.parse_args(sys.argv[1:])
    server_address = namespace.addr
    server_port = namespace.port
    client_name = namespace.name
    client_password = namespace.password

    if not 1023 < server_port < 65536:
        client_log.critical(
            f'You run client with wrong port: {server_port}. '
            f'Allowed port are from 1024 till 65535. Bye.')
        sys.exit(1)

    return server_address, server_port, client_name, client_password


def main():
    server_address, server_port, client_name, client_password = arg_parser()
    client_log.debug('Args loaded')

    # Create client app
    client_app = QApplication(sys.argv)

    # If app run without name ask it via dialog
    start_dialog = UserNameDialog()
    if not client_name or not client_password:
        client_app.exec_()

        if start_dialog.ok_pressed:
            client_name = start_dialog.client_name.text()
            client_password = start_dialog.client_password.text()
            client_log.debug(f'Using USERNAME = {client_name}, PASSWD = {client_password}.')
        else:
            exit(0)

    client_log.info(
        f'Client run with: server address: {server_address}, '
        f'port: {server_port}, client name: {client_name}')

    # Get keys from file if there is no one, generate it
    dir_path = os.path.dirname(os.path.realpath(__file__))
    key_file = os.path.join(dir_path, f'{client_name}.key')
    if not os.path.exists(key_file):
        keys = RSA.generate(2048, os.urandom)
        with open(key_file, 'wb') as key:
            key.write(keys.export_key())
    else:
        with open(key_file, 'rb') as key:
            keys = RSA.import_key(key.read())

    client_log.debug("Keys successfully loaded.")

    # Create DB object
    database = ClientDatabase(client_name)

    # Create transport object and run it
    try:
        transport = ClientTransport(server_address, server_port, database, client_name, client_password, keys)
        client_log.debug('Transport is ready')
    except ServerError as error:
        message = QMessageBox()
        message.critical(start_dialog, 'Server Error', error.text)
        exit(1)
    transport.setDaemon(True)
    transport.start()

    # Delete dialog
    del start_dialog

    # Create GUI
    main_window = ClientMainWindow(database, transport, keys)
    main_window.make_connection(transport)
    main_window.setWindowTitle(f'Simple Chat - {client_name}')
    client_app.exec_()

    # Close transport when GUI is closed
    transport.transport_shutdown()
    transport.join()


if __name__ == '__main__':
    main()



import os
import sys
import inspect

sys.path.append(os.path.join(os.getcwd(), '..'))
import logging
from logs import client_log_config, server_log_config


def log(func_to_logging):
    """Decorator function"""
    def wrap(*args, **kwargs):
        logger_name = 'server_log' if 'server.py' in sys.argv[0] else 'client_log'
        logger = logging.getLogger(logger_name)
        f = func_to_logging(*args, **kwargs)

        logger.debug(
            f'{func_to_logging.__name__} was called with params {args}, {kwargs} '
            f'from module {func_to_logging.__module__} '
            f'from function {inspect.stack()[1][3]}'
        )
        return f
    return wrap




import os

from Crypto.Cipher import PKCS1_OAEP
from Crypto.PublicKey import RSA
from PyQt5.QtWidgets import QMainWindow, qApp, QMessageBox, QApplication
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QBrush, QColor
from PyQt5.QtCore import pyqtSlot, Qt
import sys
import json
import logging

# sys.path.append('../')
from main_window_conv import Ui_MainClientWindow
from add_contact import AddContactDialog
from del_contact import DelContactDialog
from client_database import ClientDatabase
from transport import ClientTransport
from common.errors import ServerError

sys.path.append(os.path.join(os.getcwd(), '../'))
client_log = logging.getLogger('client_log')


class ClientMainWindow(QMainWindow):
    def __init__(self, database, transport, keys):
        super().__init__()

        self.database = database
        self.transport = transport

        self.decrypter = PKCS1_OAEP.new(keys)

        self.ui = Ui_MainClientWindow()
        self.ui.setupUi(self)

        # Exit button
        self.ui.menu_exit.triggered.connect(qApp.exit)

        # Send message button
        self.ui.btn_send.clicked.connect(self.send_message)

        # Add contact
        self.ui.btn_add_contact.clicked.connect(self.add_contact_window)
        self.ui.menu_add_contact.triggered.connect(self.add_contact_window)

        # Delete contact
        self.ui.btn_remove_contact.clicked.connect(self.delete_contact_window)
        self.ui.menu_del_contact.triggered.connect(self.delete_contact_window)

        # Additional attributes
        self.contacts_model = None
        self.history_model = None
        self.messages = QMessageBox()
        self.current_chat = None
        self.current_chat_key = None
        self.encryptor = None
        self.ui.list_messages.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.ui.list_messages.setWordWrap(True)

        # Double click by user list
        self.ui.list_contacts.doubleClicked.connect(self.select_active_user)

        self.clients_list_update()
        self.set_disabled_input()
        self.show()

    def set_disabled_input(self):
        self.ui.label_new_message.setText('To select receiver - double click on it')
        self.ui.text_message.clear()
        if self.history_model:
            self.history_model.clear()

        self.ui.btn_clear.setDisabled(True)
        self.ui.btn_send.setDisabled(True)
        self.ui.text_message.setDisabled(True)

        self.encryptor = None
        self.current_chat = None
        self.current_chat_key = None

    def history_list_update(self):
        list_messages = sorted(self.database.get_history(self.current_chat),
                               key=lambda item: item[3])
        if not self.history_model:
            self.history_model = QStandardItemModel()
            self.ui.list_messages.setModel(self.history_model)

        self.history_model.clear()

        length = len(list_messages)
        start_index = 0
        if length > 20:
            start_index = length - 20

        for i in range(start_index, length):
            item = list_messages[i]
            if item[1] == 'in':
                mess = QStandardItem(f'Inbox {item[3].replace(microsecond=0)}:\n {item[2]}')
                mess.setEditable(False)
                mess.setBackground(QBrush(QColor(255, 213, 213)))
                mess.setTextAlignment(Qt.AlignLeft)
                self.history_model.appendRow(mess)
            else:
                mess = QStandardItem(f'Sent {item[3].replace(microsecond=0)}:\n {item[2]}')
                mess.setEditable(False)
                mess.setTextAlignment(Qt.AlignRight)
                mess.setBackground(QBrush(QColor(204, 255, 204)))
                self.history_model.appendRow(mess)
        self.ui.list_messages.scrollToBottom()

    def select_active_user(self):
        self.current_chat = self.ui.list_contacts.currentIndex().data()
        self.set_active_user()

    def set_active_user(self):
        try:
            self.current_chat_key = self.transport.key_request(
                self.current_chat)
            client_log.debug(f'Received public key for {self.current_chat}')
            if self.current_chat_key:
                self.encryptor = PKCS1_OAEP.new(
                    RSA.import_key(self.current_chat_key))
        except (OSError, json.JSONDecodeError):
            self.current_chat_key = None
            self.encryptor = None
            client_log.debug(f'Cannot get key for {self.current_chat}')

        if not self.current_chat_key:
            self.messages.warning(
                self, 'Error', 'There is a cipher key for this user.')
            return

        self.ui.label_new_message.setText(f'Enter a message to {self.current_chat}:')
        self.ui.btn_clear.setDisabled(False)
        self.ui.btn_send.setDisabled(False)
        self.ui.text_message.setDisabled(False)

        self.history_list_update()

    def clients_list_update(self):
        contacts_list = self.database.get_contacts()
        self.contacts_model = QStandardItemModel()
        for i in sorted(contacts_list):
            item = QStandardItem(i)
            item.setEditable(False)
            self.contacts_model.appendRow(item)
        self.ui.list_contacts.setModel(self.contacts_model)

    def add_contact_window(self):
        global select_dialog
        select_dialog = AddContactDialog(self.transport, self.database)
        select_dialog.btn_ok.clicked.connect(lambda: self.add_contact_action(select_dialog))
        select_dialog.show()

    def add_contact_action(self, item):
        new_contact = item.selector.currentText()
        self.add_contact(new_contact)
        item.close()

    def add_contact(self, new_contact):
        try:
            self.transport.add_contact(new_contact)
        except ServerError as err:
            self.messages.critical(self, 'Server error', err.text)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Error', 'Connection lost')
                self.close()
            self.messages.critical(self, 'Error', 'Connection timeout')
        else:
            self.database.add_contact(new_contact)
            new_contact = QStandardItem(new_contact)
            new_contact.setEditable(False)
            self.contacts_model.appendRow(new_contact)
            client_log.info(f'New contact {new_contact} added')
            self.messages.information(self, 'Success', 'New contact added.')

    def delete_contact_window(self):
        global remove_dialog
        remove_dialog = DelContactDialog(self.database)
        remove_dialog.btn_ok.clicked.connect(lambda: self.delete_contact(remove_dialog))
        remove_dialog.show()

    def delete_contact(self, item):
        selected = item.selector.currentText()
        try:
            self.transport.remove_contact(selected)
        except ServerError as err:
            self.messages.critical(self, 'Server error', err.text)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Error', 'Connection lost')
                self.close()
            self.messages.critical(self, 'Error', 'Connection timeout')
        else:
            self.database.del_contact(selected)
            self.clients_list_update()
            client_log.info(f'Contact {selected} deleted')
            self.messages.information(self, 'Success', 'Contact deleted')
            item.close()

            if selected == self.current_chat:
                self.current_chat = None
                self.set_disabled_input()

    def send_message(self):
        message_text = self.ui.text_message.toPlainText()
        self.ui.text_message.clear()
        if not message_text:
            return
        try:
            self.transport.send_message(self.current_chat, message_text)
        except ServerError as err:
            self.messages.critical(self, 'Error', err.text)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Error', 'Connection lost!')
                self.close()
            self.messages.critical(self, 'Error', 'Connection timeout!')
        except (ConnectionResetError, ConnectionAbortedError):
            self.messages.critical(self, 'Error', 'Connection lost!')
            self.close()
        else:
            self.database.save_message(self.current_chat, 'out', message_text)
            client_log.debug(f'Sent message to {self.current_chat}: {message_text}')
            self.history_list_update()

    @pyqtSlot(str)
    def message(self, sender):
        if sender == self.current_chat:
            self.history_list_update()
        else:
            if self.database.check_contact(sender):
                if self.messages.question(self, 'New message',
                                          f'Got new message from {sender}, '
                                          f'Do you want open this chat?', QMessageBox.Yes,
                                          QMessageBox.No) == QMessageBox.Yes:
                    self.current_chat = sender
                    self.set_active_user()
            else:
                print('NO')
                if self.messages.question(self, 'New message',
                              f'Got new message from {sender}.\n '
                              f'This user is not your contact.\n'
                              f'Do you want to add it and open this chat?',
                              QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
                    self.add_contact(sender)
                    self.current_chat = sender
                    self.set_active_user()

    @pyqtSlot()
    def connection_lost(self):
        self.messages.warning(self, 'Error', 'Connection lost. ')
        self.close()

    @pyqtSlot()
    def sig_205(self):
        if self.current_chat and not self.database.check_user(
                self.current_chat):
            self.messages.warning(
                self,
                'Sorry',
                'But your interlocutor was deleted from server accidentally.')
            self.set_disabled_input()
            self.current_chat = None
        self.clients_list_update()

    def make_connection(self, trans_obj):
        trans_obj.new_message.connect(self.message)
        trans_obj.connection_lost.connect(self.connection_lost)
        trans_obj.message_205.connect(self.sig_205)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    database = ClientDatabase('test1')
    transport = ClientTransport('127.0.0.1', 7777, database, 'test1')
    window = ClientMainWindow(database, transport)
    sys.exit(app.exec_())



import dis
from pprint import pprint


class ClientVerifier(type):
    def __init__(cls, cls_name, bases, cls_dict):
        forbidden_commands = ('accept', 'listen', 'socket')
        methods = []
        # Get methods which used in classes functions
        for func in cls_dict:
            try:
                instr = dis.get_instructions(cls_dict[func])
            except TypeError:
                pass
            else:
                for i in instr:
                    if i.opname == 'LOAD_GLOBAL':
                        if i.argval not in methods:
                            methods.append(i.argval)

        for command in forbidden_commands:
            if command in methods:
                raise TypeError(f'Method {command} does not allowed')

        if 'get_message' in methods or 'send_json_message' in methods:
            pass
        else:
            raise TypeError('There is no functions working with sockets now')
        super().__init__(cls_name, bases, cls_dict)


class ServerVerifier(type):
    def __init__(cls, cls_name, bases, cls_dict):
        forbidden_commands = ('connect',)
        # Get methods which used in classes functions - get with LOAD_GLOBAL
        global_methods = []
        # Get methods wrapped by decorators - get with LOAD_METHOD
        methods = []
        # Get attributes which used in classes functions - get with LOAD_ATTR
        attrs = []

        for func in cls_dict:
            try:
                instr = dis.get_instructions(cls_dict[func])
            except TypeError:
                pass
            else:
                for i in instr:
                    if i.opname == 'LOAD_GLOBAL':
                        if i.argval not in global_methods:
                            global_methods.append(i.argval)
                    elif i.opname == 'LOAD_METHOD':
                        if i.argval not in methods:
                            methods.append(i.argval)
                    elif i.opname == 'LOAD_ATTR':
                        if i.argval not in attrs:
                            attrs.append(i.argval)

        # print(20 * '-', 'methods', 20 * '-')
        # pprint(global_methods)
        # print(20 * '-', 'methods_2', 20 * '-')
        # pprint(methods)
        # print(20 * '-', 'attrs', 20 * '-')
        # pprint(attrs)
        # print(50 * '-')

        for command in forbidden_commands:
            if command in methods:
                raise TypeError(f'Method {command} does not allowed')

        if not ('SOCK_STREAM' in global_methods and 'AF_INET' in global_methods):
            raise TypeError('Incorrect socket initialisation')
        super().__init__(cls_name, bases, cls_dict)



import logging

server_log = logging.getLogger('server_log')


class Port:
    def __set__(self, instance, value):
        if not 1023 < int(value) < 65535:
            m = f'Incorrect port value {value}. Port can be only a number between 1024 and 65535.'
            server_log.critical(m)
            print(m)
            exit(1)
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name



import os
import sys

sys.path.append('../')
import logging
from logging import handlers

DIR = f'{os.getcwd()}/logs' if __name__ != '__main__' else f'{os.getcwd()}'

server_log = logging.getLogger('server_log')
server_log.setLevel(logging.DEBUG)

formatter = logging.Formatter("%(asctime)s %(levelname)-10s  %(module)15s  %(message)s")

file_handler = logging.handlers.TimedRotatingFileHandler(f'{DIR}/logging/server_log.log',
                                                         when='D', interval=1,  encoding='utf-8')
file_handler.setLevel(logging.DEBUG)
file_handler.setFormatter(formatter)

server_log.addHandler(file_handler)


if __name__ == '__main__':
    server_log.debug('Debug msg')
    server_log.info('Info msg')
    server_log.warning('Warning msg')
    server_log.error('Error msg')
    server_log.critical(f'Crit msg')
